@@ -404,6 +404,9 @@ class NeuralNetwork:
         _hidden_size: int = _num_attention_heads * np.random.randint(low=100, high=1000)
         return dict(model_type=_model_type,
                     model_name=TRANSFORMER.get(_model_type),
+                    epoch=np.random.randint(low=3, high=20),
+                    batch_size=int(np.random.choice(a=HappyLearningUtils().geometric_progression(n=8))),
+                    learning_rate=np.random.uniform(low=0.00001, high=0.4),
                     adafactor_beta1=np.random.uniform(low=0.0, high=1),
                     adafactor_clip_threshold=np.random.uniform(low=0.001, high=1),
                     adafactor_decay_rate=np.random.uniform(low=-0.001, high=0.999),
@@ -421,10 +424,12 @@ class NeuralNetwork:
                     attention_probs_dropout_prob=np.random.uniform(low=0.05, high=0.95),
                     hidden_size=_hidden_size,
                     hidden_dropout_prob=np.random.uniform(low=0.05, high=0.95),
+                    gradient_accumulation_steps=np.random.randint(low=1, high=3),
                     initializer_range=np.random.uniform(low=0.05, high=0.95),
                     layer_norm_eps=np.random.uniform(low=0.05, high=0.5),
                     num_attention_heads=_num_attention_heads,
                     num_hidden_layers=np.random.randint(low=2, high=20),
+                    warmup_ratio=np.random.uniform(low=0.03, high=0.15),
                     optimizer=np.random.choice(a=['AdamW', 'Adafactor']),
                     scheduler=np.random.choice(a=['constant_schedule', 'constant_schedule_with_warmup', 'linear_schedule_with_warmup', 'cosine_schedule_with_warmup', 'cosine_with_hard_restarts_schedule_with_warmup', 'polynomial_decay_schedule_with_warmup']),
                     polynomial_decay_schedule_lr_end=np.random.uniform(low=1e-8, high=1e-4),
